@@ -28,7 +28,8 @@
           ,@body))
 
 (defun projectile-test-should-root-in (root directory)
-  (let ((projectile-project-root-cache (make-hash-table :test 'equal)))
+  (let ((projectile-project-root-cache (make-hash-table :test 'equal))
+        (projectile-cached-project-root nil))
     (should (equal (file-truename (file-name-as-directory root))
                    (let ((default-directory
                            (expand-file-name
@@ -115,9 +116,9 @@
            (projectile-project-name () "project")
            (projectile-ignored-files-rel () ())
            (projectile-ignored-directories-rel () ()))
-          (let* ((file-names '("foo.c" "foo.o" "foo.so" "foo.o.gz"))
+          (let* ((file-names '("foo.c" "foo.o" "foo.so" "foo.o.gz" "foo.tar.gz" "foo.tar.GZ"))
                  (files (mapcar 'projectile-expand-root file-names)))
-            (let ((projectile-globally-ignored-file-suffixes '(".o" ".so")))
+            (let ((projectile-globally-ignored-file-suffixes '(".o" ".so" ".tar.gz")))
               (should (equal (projectile-remove-ignored files)
                              (mapcar 'projectile-expand-root
                                      '("foo.c" "foo.o.gz"))))))))
@@ -136,16 +137,14 @@
 
 (ert-deftest projectile-add-unignored-files-no-vcs ()
   (noflet ((projectile-project-vcs () 'none))
-    ;; Fail when no VCS command for ignored files is found
+    ;; on an unsupported VCS we simply return the list of globally unignored files
     (let ((projectile-globally-unignored-files '("unignored-file")))
-      (should-error (projectile-add-unignored '("file"))))
-    ;; But only fail if unignored files are requested
-    (let (projectile-globally-unignored-files)
       (should (equal (projectile-add-unignored '("file")) '("file"))))))
 
 (ert-deftest projectile-add-unignored-directories ()
   (noflet ((projectile-project-vcs () 'git)
-           (projectile-get-repo-ignored-files () '("path/unignored-file")))
+           (projectile-get-repo-ignored-files () '("path/unignored-file"))
+           (projectile-get-repo-ignored-directory (dir) (list (concat dir "unignored-file"))))
     (let ((projectile-globally-unignored-directories '("path")))
       (should (equal (projectile-add-unignored '("file"))
                      '("file" "path/unignored-file")))
@@ -197,20 +196,12 @@
       (should (equal '("a/b/c" "a/d/e") (projectile-dir-files "a/"))))))
 
 (ert-deftest projectile-test-setup-hook-functions-projectile-mode ()
-  (projectile-mode 1)
-  (should (and (memq 'projectile-cache-files-find-file-hook find-file-hook)
-               (memq 'projectile-cache-projects-find-file-hook find-file-hook)))
-  (projectile-mode -1)
-  (should (and (not (memq 'projectile-cache-files-find-file-hook find-file-hook))
-               (not (memq 'projectile-cache-projects-find-file-hook find-file-hook)))))
-
-(ert-deftest projectile-test-setup-hook-functions-projectile-global-mode ()
-  (projectile-global-mode 1)
-  (should (and (memq 'projectile-cache-files-find-file-hook find-file-hook)
-               (memq 'projectile-cache-projects-find-file-hook find-file-hook)))
-  (projectile-global-mode -1)
-  (should (and (not (memq 'projectile-cache-files-find-file-hook find-file-hook))
-               (not (memq 'projectile-cache-projects-find-file-hook find-file-hook)))))
+  (noflet ((projectile-cleanup-known-projects () nil)
+           (projectile-discover-projects-in-search-path () nil))
+    (projectile-mode 1)
+    (should (memq 'projectile-find-file-hook-function find-file-hook))
+    (projectile-mode -1)
+    (should (not (memq 'projectile-find-file-hook-function find-file-hook)))))
 
 (ert-deftest projectile-test-relevant-known-projects ()
   (let ((projectile-known-projects '("/path/to/project1" "/path/to/project2")))
@@ -469,6 +460,7 @@
          "project/file4.el")
       (cd "project")
       (let ((projectile-projects-cache (make-hash-table :test #'equal))
+            (projectile-projects-cache-time (make-hash-table :test #'equal))
             (projectile-enable-caching t))
         (puthash (projectile-project-root)
                  '("file1.el")
@@ -491,6 +483,35 @@
               (should (member f (gethash (projectile-project-root)
                                          projectile-projects-cache))))))))))
 
+(ert-deftest projectile-test-cache-expiring ()
+  "Ensure that we update the cache if it's expired."
+  (projectile-test-with-sandbox
+    (projectile-test-with-files
+     ("project/"
+      "project/.projectile"
+      "project/file1.el"
+      "project/file2.el")
+     (cd "project")
+     (let ((projectile-projects-cache (make-hash-table :test #'equal))
+           (projectile-projects-cache-time (make-hash-table :test #'equal))
+           (projectile-enable-caching t)
+           (projectile-files-cache-expire 10))
+       ;; Create a stale cache with only one file in it.
+       (puthash (projectile-project-root)
+                '("file1.el")
+                projectile-projects-cache)
+       (puthash (projectile-project-root)
+                0 ;; Cached 1st of January 1970.
+                projectile-projects-cache-time)
+
+       (noflet ((projectile-project-root () (file-truename default-directory))
+                (projectile-project-vcs () 'none))
+               ;; After listing all the files, the cache should have been updated.
+               (projectile-current-project-files)
+               (dolist (f '("file1.el" "file2.el"))
+                 (should (member f (gethash (projectile-project-root)
+                                            projectile-projects-cache)))))))))
+
 (ert-deftest projectile-test-old-project-root-gone ()
   "Ensure that we don't cache a project root if the path has changed."
   (projectile-test-with-sandbox
@@ -527,8 +548,8 @@
 	      (current-prefix-arg '-)
 	      (sym (cadr test)))
 	  (noflet ((projectile-project-vcs () 'git)
-		   (read-string (prompt initial &rest args)
-				(if (should (equal sym initial)) initial))
+               (read-string (prompt initial-input history default-value &rest args)
+                            (if (should (equal sym default-value)) default-value))
 		   (vc-git-grep (regexp files dir)
 				(progn (should (equal sym regexp))
 				       (should (equal (car (last test)) files))
@@ -538,13 +559,6 @@
 		(re-search-forward sym)
 		(projectile-grep nil ?-)))))))))
 
-;;;;;;;;; fresh tests
-
-(ert-deftest projectile-clear-known-projects ()
-  (let ((projectile-known-projects '("one" "two" "three")))
-    (projectile-clear-known-projects)
-    (should (null projectile-known-projects))))
-
 (ert-deftest projectile-switch-project-no-projects ()
   (let ((projectile-known-projects nil))
     (should-error (projectile-switch-project))))
@@ -594,6 +608,7 @@
                        "src/some_module/same_name.c"
                        "include1/same_name.h"
                        "include1/test1.h"
+                       "include1/test1.h~"
                        "include1/test2.h"
                        "include1/test1.hpp"
                        "include2/some_module/same_name.h"
@@ -607,7 +622,7 @@
                        "include2/test1.service.spec.js"
                        "include1/test2.js"
                        "include2/test2.js")))
-    
+
     (should (equal '("include1/test1.h" "include2/test1.h")
                    (projectile-get-other-files "src/test1.c" source-tree)))
     (should (equal '("include1/test1.h" "include2/test1.h" "include1/test1.hpp")
@@ -636,7 +651,8 @@
 
 (ert-deftest projectile-test-compilation-directory ()
   (defun helper (project-root rel-dir)
-    (noflet ((projectile-project-root () project-root))
+    (noflet ((projectile-project-root () project-root)
+             (projectile-project-type () 'generic))
             (let ((projectile-project-compilation-dir rel-dir))
               (projectile-compilation-dir))))
 
@@ -645,6 +661,38 @@
   (should (equal "/root/build/" (helper "/root/" "./build")))
   (should (equal "/root/local/build/" (helper "/root/" "local/build"))))
 
+(ert-deftest projectile-test-compilation-directory-with-default-directory ()
+  (projectile-register-project-type 'default-dir-project '("file.txt")
+                                    :compilation-dir "build")
+  (defun helper (project-root &optional rel-dir)
+    (noflet ((projectile-project-root () project-root)
+             (projectile-project-type () 'default-dir-project))
+            (if (null rel-dir)
+                (projectile-compilation-dir)
+              (let ((projectile-project-compilation-dir rel-dir))
+                (projectile-compilation-dir)))))
+
+  (should (equal "/root/build/"       (helper "/root/")))
+  (should (equal "/root/buildings/"   (helper "/root/" "buildings"))))
+
+(ert-deftest projectile-detect-project-type-of-rails-like-npm-test ()
+  (projectile-test-with-sandbox
+   (projectile-test-with-files
+    ("project/"
+     "project/Gemfile"
+     "project/app/"
+     "project/lib/"
+     "project/db/"
+     "project/config/"
+     "project/spec/"
+     "project/package.json"
+     )
+    (let ((projectile-indexing-method 'native))
+      (noflet ((projectile-project-root
+                () (file-truename (expand-file-name "project/"))))
+              (should (equal 'rails-rspec
+                             (projectile-detect-project-type))))))))
+
 (ert-deftest projectile-test-dirname-matching-count ()
   (should (equal 2
                  (projectile-dirname-matching-count "src/food/sea.c"
@@ -688,37 +736,87 @@
          "project/spec/models/food/sea_spec.rb")
       (let ((projectile-indexing-method 'native))
         (noflet ((projectile-project-type () 'rails-rspec)
-                 (projectile-project-root
-                  () (file-truename (expand-file-name "project/"))))
+                 (projectile-project-root () (file-truename (expand-file-name "project/"))))
           (should (equal "app/models/food/sea.rb"
                          (projectile-find-matching-file
                           "spec/models/food/sea_spec.rb"))))))))
 
+(ert-deftest projectile-test-find-matching-test/file-custom-project ()
+  (projectile-test-with-sandbox
+   (projectile-test-with-files
+     ("project/src/foo/"
+      "project/src/bar/"
+      "project/test/foo/"
+      "project/test/bar/"
+      "project/src/foo/foo.service.js"
+      "project/src/bar/bar.service.js"
+      "project/test/foo/foo.service.spec.js"
+      "project/test/bar/bar.service.spec.js")
+     (let* ((projectile-indexing-method 'native)
+            (reg (projectile-register-project-type 'npm-project '("somefile") :test-suffix ".spec")))
+        (noflet ((projectile-project-type () 'npm-project)
+                 (projectile-project-root () (file-truename (expand-file-name "project/"))))
+          (let ((test-file (projectile-find-matching-test "src/foo/foo.service.js"))
+                (impl-file (projectile-find-matching-file "test/bar/bar.service.spec.js")))
+            (should (equal "test/foo/foo.service.spec.js" test-file))
+            (should (equal "src/bar/bar.service.js" impl-file))))))))
+
+(ert-deftest projectile-test-find-matching-test/file-custom-project-with-dirs ()
+  (projectile-test-with-sandbox
+   (projectile-test-with-files
+     ("project/source/foo/"
+      "project/source/bar/"
+      "project/spec/foo/"
+      "project/spec/bar/"
+      "project/source/foo/foo.service.js"
+      "project/source/bar/bar.service.js"
+      "project/spec/foo/foo.service.spec.js"
+      "project/spec/bar/bar.service.spec.js")
+     (let* ((projectile-indexing-method 'native)
+            (reg (projectile-register-project-type 'npm-project '("somefile")
+                                                   :test-suffix ".spec"
+                                                   :test-dir "spec/"
+                                                   :src-dir "source/")))
+        (noflet ((projectile-project-type () 'npm-project)
+                 (projectile-project-root () (file-truename (expand-file-name "project/"))))
+          (let ((test-file (projectile-find-matching-test "source/foo/foo.service.js"))
+                (impl-file (projectile-find-matching-file "spec/bar/bar.service.spec.js")))
+            (should (equal "spec/foo/foo.service.spec.js" test-file))
+            (should (equal "source/bar/bar.service.js" impl-file))))))))
+
 (ert-deftest projectile-test-exclude-out-of-project-submodules ()
   (projectile-test-with-files
-      (;; VSC root is here
-       "project/"
-       "project/.git/"
-       "project/.gitmodules"
-       ;; Current project root is here:
-       "project/web-ui/"
-       "project/web-ui/.projectile"
-       ;; VCS git submodule will return the following submodules,
-       ;; relative to current project root, 'project/web-ui/':
-       "project/web-ui/vendor/client-submodule/"
-       "project/server/vendor/server-submodule/")
-    (let ((project (file-truename (expand-file-name "project/web-ui"))))
-      (noflet ((projectile-files-via-ext-command
-                (arg) (when (string= default-directory project)
-                        '("vendor/client-submodule"
-                          "../server/vendor/server-submodule")))
-               (projectile-project-root
-                () project))
+   (;; VSC root is here
+    "project/"
+    "project/.git/"
+    "project/.gitmodules"
+    ;; Current project root is here:
+    "project/web-ui/"
+    "project/web-ui/.projectile"
+    ;; VCS git submodule will return the following submodules,
+    ;; relative to current project root, 'project/web-ui/':
+    "project/web-ui/vendor/client-submodule/"
+    "project/server/vendor/server-submodule/")
+   (let ((project (file-truename (expand-file-name "project/web-ui"))))
+     (noflet ((projectile-files-via-ext-command
+               (arg) (when (string= default-directory project)
+                       '("vendor/client-submodule"
+                         "../server/vendor/server-submodule")))
+              (projectile-project-root
+               () project))
 
-        ;; assert that it only returns the submodule 'project/web-ui/vendor/client-submodule/'
-        (should (equal (list (expand-file-name "vendor/client-submodule/" project))
-                       (projectile-get-all-sub-projects project)))))))
+       ;; assert that it only returns the submodule 'project/web-ui/vendor/client-submodule/'
+       (should (equal (list (expand-file-name "vendor/client-submodule/" project))
+                      (projectile-get-all-sub-projects project)))))))
+
+(ert-deftest projectile-test-configure-command-for-generic-project-type ()
+  (noflet ((projectile-default-configure-command (x) nil)
+           (projectile-project-type () 'generic))
+    (let ((configure-command (projectile-configure-command "fsdf")))
+      (should (equal nil configure-command)))))
 
 ;; Local Variables:
 ;; indent-tabs-mode: nil
 ;; End:
+
+;;; projectile-test.el ends here
